@@ -46,6 +46,7 @@ use std::iter::Iterator;
 use std::mem;
 use std::ops::Deref;
 use std::fmt::{self, Debug, Error};
+use std::hash::{Hash, Hasher};
 use self::OptionBool::*;
 
 /// The `OptionBool` type, a space-efficient Option<bool> replacement
@@ -71,7 +72,7 @@ const OB_SOME_TRUE_REF : &'static Option<bool> = &OB_SOME_TRUE;
 const OB_SOME_FALSE_REF : &'static Option<bool> = &OB_SOME_FALSE;
 const OB_NONE_REF : &'static Option<bool> = &OB_NONE;
 
-/// We can deref-coerce to Option<bool>
+/// We can deref-coerce to `Option<bool>`
 impl Deref for OptionBool {
 	type Target = Option<bool>;
 
@@ -260,7 +261,6 @@ impl OptionBool {
 		}
 	}
 	
-	
 	/// Returns the contained bool or a computed default.
     ///
     /// # Examples
@@ -280,8 +280,8 @@ impl OptionBool {
 		}
 	}
 	
-	/// Maps an OptionBool to an Option<U> by applying the function over the
-	/// contained bool.
+	/// Maps an `OptionBool` to an `Option<U>` by applying the function
+	/// over the contained bool.
 	///
 	/// # Examples
 	///
@@ -302,12 +302,12 @@ impl OptionBool {
 		}
 	}
 	
-	/// Maps an OptionBool to another OptionBool by applying the function over
-	/// the contained bool.
+	/// Maps an `OptionBool` to another `OptionBool` by applying the
+	/// function over the contained bool.
 	///
 	/// # Examples
 	///
-	/// Invert the contained bool
+	/// Invert the contained `bool`
 	///
 	/// ```
 	///# use optional::OptionBool;
@@ -326,7 +326,8 @@ impl OptionBool {
 		}
 	}
 	
-	/// Maps a value to a U by applying the function or return a default U.
+	/// Maps a value to a `U` by applying the function or return a 
+	/// default `U`.
 	///
 	/// # Examples
 	///
@@ -347,8 +348,8 @@ impl OptionBool {
 		}
 	}
 	
-	/// Maps a value to a U by applying the function or return a computed 
-	/// default.
+	/// Maps a value to a `U` by applying the function or return a 
+	/// computed default.
 	///
 	/// # Examples
 	///
@@ -367,8 +368,8 @@ impl OptionBool {
 		}
 	}
 	
-	/// Transforms the `OptionBool` into a `Result<bool, E>`, mapping `Some`X to
-    /// `Ok(`X`)` and `None` to `Err(err)`.
+	/// Transforms the `OptionBool` into a `Result<bool, E>`, mapping 
+	/// `Some`X to `Ok(`X`)` and `None` to `Err(err)`.
     ///
     /// # Examples
     ///
@@ -626,6 +627,40 @@ impl Debug for OptionBool {
 	}
 }
 
+pub struct IterBool { o: OptionBool }
+
+impl Iterator for IterBool {
+	type Item = bool;
+	
+	#[inline]
+	fn next(&mut self) -> Option<bool> {
+		self.o.take()
+	}
+}
+
+/// IntoIterator works as expected
+///
+/// # Examples
+///
+/// ```
+///# use optional::OptionBool;
+/// let mut pass : bool = false;
+/// for b in OptionBool::SomeTrue { pass = b; }
+/// assert!(pass);
+///
+/// for b in OptionBool::None { assert!(false); }
+/// ```
+impl IntoIterator for OptionBool {
+	type Item = bool;
+	type IntoIter = IterBool;
+	
+	 #[inline]
+    fn into_iter(self) -> IterBool {
+		IterBool{ o: self }
+    }
+}
+
+/// OptionBool defaults to None.
 impl Default for OptionBool {
 	#[inline]
 	fn default() -> OptionBool { None }
@@ -781,11 +816,104 @@ impl Noned for f64 {
 	fn get_none() -> f64 { std::f64::NAN }
 }
 
-/// An Option<T>-like structure that takes only as much space as the enclosed
-/// value, at the cost of removing one particular None value from the value 
-/// domain (see Noned)
+///Equality within Optioned
+pub trait OptEq {
+	#[inline]
+	fn opt_eq(&self, other: &Self) -> bool;
+}
+
+impl OptEq for u8 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for u16 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for u32 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for u64 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for usize { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+
+impl OptEq for i8 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for i16 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for i32 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for i64 { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+impl OptEq for isize { fn opt_eq(&self, other: &Self) -> bool { self == other } }
+
+impl OptEq for f32 { 
+	fn opt_eq(&self, other: &Self) -> bool { 
+		if self.is_nan() { other.is_nan() } else { self == other } 
+	}
+}
+impl OptEq for f64 { 
+	fn opt_eq(&self, other: &Self) -> bool { 
+		if self.is_nan() { other.is_nan() } else { self == other } 
+	}
+}
+
+///Ordering within Optioned
+pub trait OptOrd {
+	#[inline]
+	fn opt_cmp(&self, other: &Self) -> Ordering;
+}
+
+#[inline]
+fn _opt_cmp<T: Ord + Copy + Noned>(a: &T, b: &T) -> Ordering {
+	if a.is_none() {
+		if b.is_none() { Ordering::Equal } else { Ordering::Less }
+	} else {
+		if b.is_none() { Ordering::Greater } else { a.cmp(b) }
+	}
+}
+
+#[inline]
+fn _opt_cmp_part<T: PartialOrd + Copy + Noned>(a: &T, b: &T) -> Ordering {
+	if a.is_none() {
+		if b.is_none() { Ordering::Equal } else { Ordering::Less }
+	} else {
+		if b.is_none() { Ordering::Greater } else { a.partial_cmp(b).unwrap() }
+	}
+}
+
+impl OptOrd for u8 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp(self, other) }
+}
+impl OptOrd for u16 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp(self, other) }
+}
+impl OptOrd for u32 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp(self, other) }
+}
+impl OptOrd for u64 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp(self, other) }
+}
+impl OptOrd for usize {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp(self, other) }
+}
+
+impl OptOrd for i8 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { self.cmp(other) }
+}
+impl OptOrd for i16 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { self.cmp(other) }
+}
+impl OptOrd for i32 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { self.cmp(other) }
+}
+impl OptOrd for i64 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { self.cmp(other) }
+}
+impl OptOrd for isize {
+	fn opt_cmp(&self, other: &Self) -> Ordering { self.cmp(other) }
+}
+
+impl OptOrd for f32 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp_part(self, other) }
+}
+impl OptOrd for f64 {
+	fn opt_cmp(&self, other: &Self) -> Ordering { _opt_cmp_part(self, other) }
+}
+
+
+/// An `Option<T>`-like structure that takes only as much space as the enclosed
+/// value, at the cost of removing one particular `None` value from the value 
+/// domain (see `Noned`)
 #[derive(Copy, Clone)]
-pub struct Optioned<T: Noned + Sized + Copy> { value: T }
+pub struct Optioned<T: Noned + Copy> { value: T }
 
 /// Equality works as usual.
 ///
@@ -796,20 +924,37 @@ pub struct Optioned<T: Noned + Sized + Copy> { value: T }
 /// assert_eq!(some(1u8), some(1u8));
 /// assert_eq!(none::<u32>(), none::<u32>());
 /// ```
-impl<T> PartialEq for Optioned<T> where T: PartialEq + Noned + Sized + Copy {
+impl<T> PartialEq for Optioned<T> where T: OptEq + Noned + Copy {
 	#[inline]
 	fn eq(&self, other: &Self) -> bool {
-		if self.is_none() { 
-			other.is_none() // null check to ensure none == none
-		} else { 
-			&self.value == &other.value 
-		}
+		self.value.opt_eq(&other.value)
 	}
 }
 
-impl<T> Eq for Optioned<T> where T: PartialEq + Noned + Sized + Copy + Eq {}
+impl<T> Eq for Optioned<T> where T: OptEq + Noned + Copy {}
 
-impl<T: Noned + Sized + Copy> Optioned<T> {
+impl<T> PartialOrd for Optioned<T> where T: PartialEq + OptEq + OptOrd + Noned + Copy {
+	#[inline]
+	fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+		Some(self.value.opt_cmp(&other.value))
+	}
+}
+
+impl<T> Ord for Optioned<T> where T: Eq + OptEq + OptOrd + Noned + Copy {
+	#[inline]
+	fn cmp(&self, other: &Self) -> Ordering {
+		self.value.opt_cmp(&other.value)
+	}
+}
+
+impl<T> Hash for Optioned<T> where T: Noned + Copy + Hash {
+	#[inline]
+	fn hash<H>(&self, state: &mut H) where H: Hasher {
+		self.value.hash(state)
+	}
+}
+
+impl<T: Noned + Copy> Optioned<T> {
 	/// Create an `Optioned<T>` that is `some(t)`.
 	///
 	/// # Panics
@@ -894,18 +1039,18 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 	///
 	/// # Panics
 	///
-	/// if self is None
+	/// if self is `None`
 	///
 	/// # Examples
 	/// 
-	/// For Some(_), the corresponding value is returned.
+	/// For `Some(_)`, the corresponding value is returned.
 	///
 	/// ```
 	///# use optional::Optioned;
 	/// assert_eq!(42u8, Optioned::some(42u8).unwrap());
 	/// ```
 	/// 
-	/// On None, it panics with the given message.
+	/// On `None`, it panics with the given message.
 	///
 	/// ```should_panic
 	///# use optional::Optioned;
@@ -916,6 +1061,20 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 		self.expect("unwrap called on None")
 	}
 	
+	/// Returns the contained value, even if None.
+	///
+	/// # Examples
+	///
+	/// ```
+    ///# use optional::{some, none};
+    /// assert_eq!(-128i8, none().unpack());
+    /// assert_eq!(1u32, some(1).unpack());
+    /// ```
+	#[inline]
+	pub fn unpack(&self) -> T {
+		self.value
+	}
+	
 	/// Returns the contained value or a default.
     ///
     /// # Examples
@@ -944,7 +1103,7 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 		if self.is_none() { f() } else { self.value }
 	}
 	
-	/// Maps the Optioned to an Option<U> by applying the function over the
+	/// Maps the `Optioned` to an `Option<U>` by applying the function over the
 	/// contained value, if any.
 	///
 	/// # Examples
@@ -959,9 +1118,9 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 		if self.is_none() { Option::None } else { Option::Some(f(self.value)) }
 	}
 	
-	/// Maps the Optioned<T> to an Optioned<U> by applying the function over
+	/// Maps the `Optioned<T>` to an `Optioned<U>` by applying the function over
 	/// the contained value, if any. Requires that the result type of the
-	/// function be Noned + Sized + Copy, as other types aren't compatible with
+	/// function be `Noned + Copy`, as other types aren't compatible with
 	/// Optioned.
 	///
 	/// # Examples
@@ -972,11 +1131,11 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 	/// ```
 	#[inline]
 	pub fn map_t<U, F>(self, f: F) -> Optioned<U>
-	where F: FnOnce(T) -> U, U: Noned + Sized + Copy {
+	where F: FnOnce(T) -> U, U: Noned + Copy {
 		if self.is_none() { none() } else { some(f(self.value)) }
 	}
 	
-	/// Maps the contained value to a U by applying the function or return a 
+	/// Maps the contained value to a `U` by applying the function or return a 
 	/// default.
 	///
 	/// # Examples
@@ -990,7 +1149,7 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 		if self.is_none() { default } else { f(self.value) }
 	}
 	
-	/// Maps a value to a U by applying the function or return a computed 
+	/// Maps a value to a `U` by applying the function or return a computed 
 	/// default.
 	///
 	/// # Examples
@@ -1070,7 +1229,7 @@ impl<T: Noned + Sized + Copy> Optioned<T> {
 ///# use ::optional::some;
 /// some(std::f64::NAN); // panic!s
 /// ```
-pub fn some<T: Noned + Sized + Copy>(value: T) -> Optioned<T> {
+pub fn some<T: Noned + Copy>(value: T) -> Optioned<T> {
 	Optioned::<T>::some(value)
 }
 
@@ -1085,11 +1244,24 @@ pub fn some<T: Noned + Sized + Copy>(value: T) -> Optioned<T> {
 /// let x : Optioned<i16> = none();
 /// none::<f32>();
 /// ```
-pub fn none<T: Noned + Sized + Copy>() -> Optioned<T> {
+pub fn none<T: Noned + Copy>() -> Optioned<T> {
 	Optioned::<T>::none()
 } 
 
-impl<T: Noned + Sized + Copy + Debug> Debug for Optioned<T> {
+/// Wrap a `T` into an `Optioned<T>`, regardless of its None-ness.
+///
+/// # Examples
+///
+/// ```
+///# use optional::wrap;
+/// assert!(wrap(1u8).is_some());
+/// assert!(wrap(255u8).is_none());
+/// ```
+pub fn wrap<T: Noned + Copy>(v: T) -> Optioned<T> {
+	Optioned{ value: v }
+}
+
+impl<T: Noned + Copy + Debug> Debug for Optioned<T> {
 	fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), Error> {
 		if self.is_none() {
 			write!(f, "None")
@@ -1099,10 +1271,17 @@ impl<T: Noned + Sized + Copy + Debug> Debug for Optioned<T> {
 	}
 }
 
-#[derive(Copy, Clone)]
-pub struct OptionedIter<T: Noned + Sized + Copy> { o: Optioned<T> }
+impl<T: Noned + Copy> Default for Optioned<T> {
+	#[inline]
+	fn default() -> Optioned<T> {
+		none()
+	}
+}
 
-impl<T: Noned + Sized + Copy> Iterator for OptionedIter<T> {
+#[derive(Copy, Clone)]
+pub struct OptionedIter<T: Noned + Copy> { o: Optioned<T> }
+
+impl<T: Noned + Copy> Iterator for OptionedIter<T> {
 	type Item=T;
 	
 	#[inline]
@@ -1111,21 +1290,21 @@ impl<T: Noned + Sized + Copy> Iterator for OptionedIter<T> {
 	}
 }
 
-impl<'a, T: Noned + Sized + Copy> From<&'a Option<T>> for Optioned<T> {
+impl<'a, T: Noned + Copy> From<&'a Option<T>> for Optioned<T> {
 	#[inline]
 	fn from(o: &Option<T>) -> Optioned<T> {
 		o.map_or_else(Self::none, Self::some)
 	}
 }
 
-impl<T: Noned + Sized + Copy> From<Option<T>> for Optioned<T> {
+impl<T: Noned + Copy> From<Option<T>> for Optioned<T> {
 	#[inline]
 	fn from(o: Option<T>) -> Optioned<T> {
 		o.map_or_else(Self::none, Self::some)
 	}
 }
 
-impl<T: Noned + Sized + Copy> Into<Option<T>> for Optioned<T> {
+impl<T: Noned + Copy> Into<Option<T>> for Optioned<T> {
 	#[inline]
 	fn into(self) -> Option<T> { self.as_option() }
 }
